package org.evelsoft.test.it.performance;


import javax.inject.Inject;
import javax.ejb.EJB;


import org.evelsoft.test.beans.HelloWorld;
import org.evelsoft.test.beans.HelloWorldBean;
import org.evelsoft.test.beans.HelloWorldEJBBean;
import org.evelsoft.test.beans.HelloWorldWeldBean;
import org.evelsoft.test.beans.HelloWorldBeanMethodIEJBntercepted;
import org.evelsoft.test.beans.HelloWorldBeanClassEJBIntercepted;
import org.evelsoft.test.beans.HelloWorldBeanClassWeldIntercepted;
import org.evelsoft.test.beans.HelloWorldBeanMethodWeldIntercepted;
import org.evelsoft.test.interceptors.InterceptorCount;

import org.jboss.arquillian.api.Deployment;
import org.jboss.arquillian.junit.Arquillian;
import org.jboss.shrinkwrap.api.asset.ByteArrayAsset;
import org.jboss.shrinkwrap.api.ArchivePaths;
import org.jboss.shrinkwrap.api.ShrinkWrap;
import org.jboss.shrinkwrap.api.spec.JavaArchive;
import org.jboss.shrinkwrap.api.spec.EnterpriseArchive;
import org.junit.AfterClass;
import org.junit.Test;
import org.junit.runner.RunWith;
import static org.junit.Assert.*;

/**
 * @author wojtask9
 */
@RunWith(Arquillian.class)
public class InterceptorsPerformanceTest {

	private static int METHOD_INVOKE = 100;

	@Deployment
	public static JavaArchive createTestArchive() {
		JavaArchive jar = ShrinkWrap.create(JavaArchive.class, "test.jar").
				addPackage(HelloWorldBean.class.getPackage()).
				addPackage(InterceptorCount.class.getPackage()).
				addManifestResource("META-INF/test-beans.xml", ArchivePaths.create("beans.xml"));
				//addManifestResource("ejb-jar.xml");
		//System.out.println(jar.toString(true));
		return jar;
	}

//	@Deployment
	public static EnterpriseArchive createDeployment() {
		JavaArchive ejb = ShrinkWrap.create(JavaArchive.class, "blagh.jar").addPackage(HelloWorldBean.class.getPackage()).addManifestResource(
				new ByteArrayAsset("<beans/>".getBytes()),
				ArchivePaths.create("beans.xml"));
		return ShrinkWrap.create(EnterpriseArchive.class, "test.ear").addModule(ejb);
	}
	
	@AfterClass
	public static void printSummary(){
		System.out.println(summary.toString());
	}


	
	@EJB
	private HelloWorldBeanClassEJBIntercepted helloEJBClassIntercepted;
	@EJB
	private HelloWorldBeanMethodIEJBntercepted helloEJBMethodIntercepted;
	@EJB
	private HelloWorldBeanClassWeldIntercepted helloWeldClassIntercepted;
	@EJB
	private HelloWorldBeanMethodWeldIntercepted helloWeldMethodIntercepted;
	@EJB
	private HelloWorldEJBBean helloEJB;
	@Inject
	private HelloWorldWeldBean helloWeld;
	@Inject
	private HelloWorldBeanClassWeldIntercepted helloWeldBeanClassIntercepted;
	@Inject
	private HelloWorldBeanMethodWeldIntercepted helloWeldBeanMethodIntercepted;
	private HelloWorldBean helloPojo = new HelloWorldBean();
	
	private static StringBuilder summary = new StringBuilder();

	@Test
	public void testEJBClassIntercepted() {
		//methods incrementNumber + sayHallo
		int expected = METHOD_INVOKE + 1;
		String result = this.testInterceptor(helloEJBClassIntercepted, expected);
		summary.append(result).append(" [@EJB EJB Class Interceptor]").append("\n");
	}

	@Test
	public void testEJBMethodIntercepted() {
		String result = this.testInterceptor(helloEJBMethodIntercepted, METHOD_INVOKE);
		summary.append(result).append(" [@EJB Method EJB-Interceptor]").append("\n");
	}

	@Test
	public void testEJBClassWeldIntercepted() {
		//methods incrementNumber + sayHallo
		int expected = METHOD_INVOKE + 1;
		String result = this.testInterceptor(helloWeldClassIntercepted, expected);
		summary.append(result).append(" [@EJB Class Weld-Interceptor]").append("\n");
	}

	@Test
	public void testEJBMethodWeldIntercepted() {
		String result = this.testInterceptor(helloWeldMethodIntercepted, METHOD_INVOKE);
		summary.append(result).append(" [@EJB Method Weld-Interceptor]").append("\n");
	}

	@Test
	public void testEJBWithoutInterceptor() {
		String result = this.testMethod(helloEJB, METHOD_INVOKE);
		summary.append(result).append(" [@EJB No Interceptor]").append("\n");
	}

	@Test
	public void testInjectWithoutInterceptor() {
		String result = this.testMethod(helloWeld, METHOD_INVOKE);
		summary.append(result).append(" [@Inject No Interceptor]").append("\n");
	}

	@Test
	public void testInjectClassIntercepted() {
		//methods incrementNumber + sayHallo
		int expected = METHOD_INVOKE + 1;
		String result = this.testInterceptor(helloWeldBeanClassIntercepted, expected);
		summary.append(result).append(" [@Inject Class Weld-Interceptor]").append("\n");
	}

	@Test
	public void testInjectMethodIntercepted() {
		String result = this.testInterceptor(helloWeldBeanMethodIntercepted, METHOD_INVOKE);
		summary.append(result).append(" [@Inject Method Weld-Interceptor]").append("\n");
	}

	@Test
	public void testPojo() {
		String result = this.testMethod(helloPojo, METHOD_INVOKE);
		summary.append(result).append(" [POJO]").append("\n");
	}
	
	
	private String testInterceptor(HelloWorld bean, int expected) {
		String result = bean.sayHelloEJB("Weld");
		assertEquals("Hello Weld", result);

		long start = System.currentTimeMillis();
		Integer res = new Integer(0);
		for (int i = 0; i < METHOD_INVOKE; i++) {
			res += bean.incrementNumber(i);
		}
		long end = System.currentTimeMillis();
		assertNotNull(res > METHOD_INVOKE);
		int interceptorCount = InterceptorCount.interceptorInvokeCount;
		InterceptorCount.interceptorInvokeCount = 0;
		assertEquals(expected, interceptorCount);

		return "This took time: " + (end - start) + "ms";
	}

	private String testMethod(HelloWorld bean, int expected) {
		String result = bean.sayHelloEJB("Weld");
		assertEquals("Hello Weld", result);

		long start = System.currentTimeMillis();
		Integer res = new Integer(0);
		for (int i = 0; i < METHOD_INVOKE; i++) {
			res += bean.incrementNumber(i);
			InterceptorCount.interceptorInvokeCount++;
		}
		long end = System.currentTimeMillis();
		assertNotNull(res > METHOD_INVOKE);
		int interceptorCount = InterceptorCount.interceptorInvokeCount;
		InterceptorCount.interceptorInvokeCount = 0;
		assertEquals(expected, interceptorCount);
		return "This took time: " + (end - start) + "ms";
	}
}
